@@ -107,6 +107,7 @@ module Database.PostgreSQL.Simple.FromField
     , PQ.Format(..)
     , pgArrayFieldParser
 
+    , optionalField
     , fromJSONField
     ) where
 
@@ -268,9 +269,20 @@ instance FromField () where
 --   compatible with type @a@.  Note that the type is not checked if
 --   the value is null, although it is inadvisable to rely on this
 --   behavior.
-instance (FromField a) => FromField (Maybe a) where
-    fromField _ Nothing = pure Nothing
-    fromField f bs      = Just <$> fromField f bs
+instance FromField a => FromField (Maybe a) where
+    fromField = optionalField fromField
+
+-- | For dealing with SQL @null@ values outside of the 'FromField' class.
+--   Alternatively, one could use 'Control.Applicative.optional',  but that
+--   also turns type and conversion errors into 'Nothing',  whereas this is
+--   more specific and turns only @null@ values into 'Nothing'.
+
+optionalField :: FieldParser a -> FieldParser (Maybe a)
+optionalField p f mv =
+    case mv of
+      Nothing -> pure Nothing
+      Just _  -> Just <$> p f mv
+{-# INLINE optionalField #-}
 
 -- | compatible with any data type,  but the value must be null
 instance FromField Null where
@@ -514,7 +526,7 @@ fromArray fieldParser typeInfo f = sequence . (parseIt <$>) <$> array delim
     fElem = f{ typeOid = typoid (typelem typeInfo) }
 
     parseIt item =
-        fieldParser f' $ if item' == "NULL" then Nothing else Just item'
+        fieldParser f' $ if item == Arrays.Plain "NULL" then Nothing else Just item'
       where
         item' = fmt delim item
         f' | Arrays.Array _ <- item = f
@@ -559,6 +571,16 @@ instance FromField JSON.Value where
 --
 -- The 'Typeable' constraint is required to show more informative
 -- error messages when parsing fails.
+--
+-- Note that @fromJSONField :: FieldParser ('Maybe' Foo)@ will return
+-- @'Nothing'@ on the json @null@ value, and return an exception on SQL @null@
+-- value.  Alternatively,  one could write @'optionalField' fromJSONField@
+-- that will return @Nothing@ on SQL @null@,  and otherwise will call
+-- @fromJSONField :: FieldParser Foo@ and then return @'Just'@ the
+-- result value,  or return its exception.  If one would
+-- like to return @Nothing@ on both the SQL @null@ and json @null@ values,
+-- one way to do it would be to write
+-- @\\f mv -> 'Control.Monad.join' '<$>' optionalField fromJSONField f mv@
 fromJSONField :: (JSON.FromJSON a, Typeable a) => FieldParser a
 fromJSONField f mbBs = do
     value <- fromField f mbBs
@@ -567,7 +589,6 @@ fromJSONField f mbBs = do
                             "JSON decoding error: " ++ err
         JSON.Success x -> pure x
 
-
 -- | Compatible with the same set of types as @a@.  Note that
 --   modifying the 'IORef' does not have any effects outside
 --   the local process on the local machine.
